%This is a function to produce a bunch of simulations using the 10x10 matrix
%It assumes you have specified the stochastic matrix model in a *.mat file.  This is done by running the simplemat.m file for example
%The simplemat.m file specifies all the info for the simulation.  Thus to run a new model, make changes to simplemat.m, save as
%   newmat.m, and run gensimests('newmat')
%This file uses no sampling error; but uses different sampling intervals 1:6 years apart

function savefilename = gensimests(filetorun,age_of_first_rep, lambda, agestruc,densdep, paramyears) 
%filetorun is the file with the matrix defined, e.g. "marshmat"
%agestruc specifies whether to include an age perturbation = zero age 1 individuals
%densdep specifies whether to include density dependence
%paramyears is the number of years in each simulated data run

%filetorun is where the lifehistory matrix is specified
%make sure updated matrix is used; just in case it's been changed
feval(filetorun,age_of_first_rep, lambda) 
load(filetorun) %loads up the saved file

%display filename and info
disp([filetorun ' ' num2str(agestruc) ' ' num2str(densdep) ' ' num2str(paramyears)]);

%this bit of code
%determines the true values of the diffusion model
%that fits the stochastic matrix model
%it based on looking at mean and var of log(N_t+tau/N_t) changes over a collection of time series
%not how it changes WITHIN a single time series.
simlen2=5000;
simlen=101;
burn=50;%allow a burnin to reach equilibrium
totalpops=zeros(simlen2,simlen);
matsize=length(stableagestruc);
for j = 1:simlen2,
   census = ones(matsize,simlen);
   census(:,1) = 100000*stableagestruc;
   for i = 2:(simlen+burn),
      %variability to add; mymvnrnd is subfunction attached below; it gives lognormally drawn multivariates
      stochmat = stochplace;
      for(ii=(1:length(evar)))
         num_es=sum(stochplace(:)==ii);
         e=mymvnrnd(num_es, 0, evar(ii), ecorr(ii));
         stochmat(stochplace==ii)=e;
      end
      if( densdep ) 
         %-------------
         %Here code can be added to modify stochmat to reflect dens dep of some sort
         %-------------
      end
      %this year's matrix is the base matrix * lognormally drawn errors
      thisyearmat = themat .* stochmat;
      tmp=thisyearmat(2:matsize,:);
      %don't allow survivorship over 1
      tmp(tmp>1)=1;
      thisyearmat(2:matsize,:)=tmp;   
      census(:,i) = thisyearmat*census(:,i-1);
   end
   %total pop is not necessarily sum(N); for lots of populations with high juvenile
   %mortality, sum(N) would be a time series tracking newborns since N(1) is very big
   %relative to N(j!=1).  In an extinction analysis, most of these newborns are relatively
   %unimportant since they'll die anyhow.  In my experience, the correct 
   %'total pop' to use is the 'individuals that reach reproductive age'.
   %Thus when you sum N, you weight N(i) by the fraction of age i individuals
   %that reach reproductive age on average.
   %weight is specified in the file that specifies the matrix
   %This terse formula sums census over the rows weighted by weight to produce a time series of total pop
   %It works since the function 'sum' sums over rows by default
   totalpops(j,:) = sum(census(:,(burn+1):end).*(weight'*ones(1,simlen)),1);
end

%calculate the correct mu and sigma2 for diffusion approximation
%these are based on the property of the diffusion model that mean(log N_t/N_0) goes to mu*t as t gets big
%and var(log N_t/N_0) goes to s2*t as t gets big
%truepar is the best random walk model parameters and correlated random walk model parameters
%'best' is not defined in a statistical sense here but rather the estimates you would come up 
%with if you looked at the var(tau) vs tau plot;
mu = mean(log(totalpops(:,simlen)./totalpops(:,1)))/(simlen-1); %same for rw and crw
s2p_rw = var(log(totalpops(:,simlen)./totalpops(:,1)))/(simlen-1);
s2p_crw = var(log(totalpops(:,simlen)./totalpops(:,20)))/(simlen-20);
s2np_crw = var(log(totalpops(:,2)./totalpops(:,1)))/(2-1)-s2p_crw;
truepar=[ mu s2p_rw s2p_crw s2np_crw];

%actual probability of 90%decline at END of time t observed
%calculated from the fraction of the simlen2 reps that hit 90% decline
trueprob90=sum(totalpops(:,2:simlen)<(0.1*totalpops(:,1)*ones(1,simlen-1)),1)/simlen2;
trueprob50=sum(totalpops(:,2:simlen)<(0.5*totalpops(:,1)*ones(1,simlen-1)),1)/simlen2;
%actual cumulative probability of 90% decline WITHIN t years
tmp=cumsum(totalpops(:,2:simlen)<(0.1*totalpops(:,1)*ones(1,simlen-1)),2);
trueprob90quasi=sum(tmp>0,1)/simlen2;
tmp=cumsum(totalpops(:,2:simlen)<(0.5*totalpops(:,1)*ones(1,simlen-1)),2);
trueprob50quasi=sum(tmp>0,1)/simlen2;

savetotalpops=totalpops; %I want to save this in a file at the end of the function

%*************************************************************************************************
%STEP 2 Calculate the performance of the estimators for the diffusion model
%*************************************************************************************************

simlen2=1000; %number of trys of replicates to do; 
burn=50; %burn = 1 is no burn
simlen = paramyears; %years per replicate

numcols=4; %number of estimators to use
straightdennis = zeros(numcols*2,simlen2); %Dennis et al 1991
kalman = zeros(numcols*2,simlen2); %Lindley 2003; Holmes 2004
slopeweight = zeros(numcols*2,simlen2); %Holmes 2001; Holmes 2004
heydecohen = zeros(numcols*2,simlen2); %not used; backwards compatibility with paramperf

matsize=length(stableagestruc);

jindex=1;
for j = 1:simlen2,
   census = ones(matsize,simlen+burn);
   census(:,1) = 100000*stableagestruc; %stable age struc defined in simplemat.m
   for i = 2:(burn+simlen),  
      if(agestruc==2 & i == (burn+2)) census(1,i-1)=0; end %perturbation means no reproduction
      stochmat = stochplace;
      for(ii=(1:length(evar)))
         num_es=sum(stochplace(:)==ii);
         e=mymvnrnd(num_es, 0, evar(ii), ecorr(ii));
         stochmat(stochplace==ii)=e;
      end
      if(densdep) 
         %-------------
         %Here code can be added to modify stochmat to reflect dens dep of some sort
         %-------------
      end
      thisyearmat = themat .* stochmat;
      tmp=thisyearmat(2:matsize,:);
      tmp(tmp>1)=1;
      thisyearmat(2:matsize,:)=tmp;
      
      census(:,i) = thisyearmat*census(:,i-1);
   end % i years
   
   totalpop = sum(census(:,(burn+1):end).*(weight'*ones(1,simlen)),1);
   %obsstage is spec-ed in simplemat.m
   observed = sum(census(:,(burn+1):end).*(obsstage'*ones(1,simlen)),1);
   noerror_observed = observed; %just saving the original
   
   %Sampling spacing
   for k = 1:4,
      sampling_sep = [1 2 4 6];            
      %the get_dennis_ests and get_kalman_ests want missing values spec-ed with -99
      %work backwards; make vec of -99 then replace non -99 values with actual vals
      observed = -99*ones(1,length(noerror_observed));
      observed(1:sampling_sep(k):end)=noerror_observed(1:sampling_sep(k):end);
      %Get parameter estimates from the simulated data
      %application of Dennis et al 1991
      dennis_ests=get_dennis_ests(observed);
      straightdennis((2*k-1):2*k,jindex) = [dennis_ests.mu; dennis_ests.s2p]; 
      
      kalman_ests=get_kalman_ests(observed);
      kalman((2*k-1):2*k,jindex) = [kalman_ests.mu; kalman_ests.s2p];
      
      %get_slope_ests can't handle the -99s; send it just the number and adjust ests for the sampling_sep
      %the slope estimator is ad hoc, but it often does better at these types of problems
      %get_slope_ests(data, how many consecutive numbers in runsum, min tau for var calc, max tau for tau calc)
      %runsum filter = [1 1 1 1], min var calc = 1, max var calc = 3 is GENERIC
      %with a 120 yr time series, I could have designed a much better slope estimator, but used a generic one seemed fairer
      limitmin = 1; limitmax = 3;
      slope_ests=get_slope_ests(noerror_observed(1:sampling_sep(k):end),[1 1 1 1],limitmin,limitmax);
      %need to correct for the fact that get_slope_ests thinks sep=1
      slope_ests.s2p = slope_ests.s2p * (limitmax-limitmin)/((1+sampling_sep(k)*(limitmax-1))-(1+sampling_sep(k)*(limitmin-1)));
      slopeweight((2*k-1):2*k,jindex) = [slope_ests.mu; slope_ests.s2p];
      
      if(mod(jindex,10)==0) disp(['samp err type ' num2str(k) ' ' num2str(jindex) ' out of ' num2str(simlen2)]); end  %display counter
   end %k types of sampling distance
   jindex=jindex+1; 
   
end %j replicates

tmp=clock;

%these are just notes about the run
notes=strvcat(datatitle, ' ', 'the matrix was', num2str(themat),' ','the variance was placed at ',...
   num2str(stochplace),char(['drawn from a mvlognormal dist with variances ' num2str(evar)]),char(['and correlation ' num2str(ecorr)]),...
   char(['number of reps ', num2str(simlen2), ' and number of years in each rep', num2str(simlen)]), char(['run on ', date, ' at ', char([num2str(tmp(4)) ':' num2str(tmp(5))]) ]),...
   ' ', 'data is in pairs.  first line is mu est, second line is sigma2 est','Sets are different amounts of lognormal sampling error as follows',...
   'sampleerror variance = 0; bias=0','sampleerror variance = .1; bias=0','sampleerror variance depends on population size',...
   'sampleerror variance = 0; bias=0','sampleerror variance = .5; bias=0','sampleerror variance depends on population size',...
   'sampleerror variance = 0; bias=0','sampleerror variance = .75; bias=0','sampleerror variance depends on population size',...
   'truepar calculated with 1000 100 year sims',...
   char(['the initial age structure was: ',census(:,1)']));

%make a unique name; 'code' is the name tag in 'filetorun'
savefilename = char([code '-' num2str(age_of_first_rep) '-' num2str(lambda) '-' date '-' char([num2str(tmp(4)) num2str(tmp(5))]) ]);
save( savefilename, 'notes', 'straightdennis', 'slopeweight', 'truepar', 'trueprob90', 'trueprob90quasi','trueprob50', 'trueprob50quasi','savetotalpops','kalman', 'heydecohen')

%subfunction
function ms = mymvnrnd(nvars, means, vars, corr)
%mymvnrnd returns nvars LOGNORMAL with means, vars, and correlation

% means is the means of your random variable
if (length(means) == 1) means = means*ones(1,nvars); end
% vars is the variances of your ransom variables
if (length(vars) == 1) vars=vars*ones(1, nvars); end
% corr is the correlation coeffient for your random variables; at this point corr must be the same for all
if(corr==1) 
   corr=.999999; %cludge. This func. doesn't like corr=1 since varcor then isn't a pos def matrix
end
temp0 = repmat(sqrt(vars),[nvars 1]) .* repmat(sqrt(vars)',[1 nvars]);
temp1 = temp0 .* (corr*ones(nvars,nvars));
temp2 = temp0 .* ((1-corr)*eye(nvars,nvars));
varcovmatrix = (temp1+temp2);
ms = mvnrnd(means, varcovmatrix,1);
ms = exp(ms);